CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  Program file name: spline.f90                                       C
C                                                                      C
C  © Tao Pang 2006                                                    C
C                                                                      C
C  Last modified: August 1, 2012                                       C
C                                                                      C
C  (1) This F90 program is created for the book, "An Introduction to   C
C      Computational Physics, 2nd Edition," written by Tao Pang and    C
C      published by Cambridge University Press on January 19, 2006.    C
C                                                                      C
C  (2) No warranties, express or implied, are made for this program.   C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
       SUBROUTINE  Int_SplineCubica(XI,FI,ldm,XO,FO,lm)
       IMPLICIT NONE

C An example of creating cubic-spline approximation of
C a discrete function fi=f(xi).
C
       INTEGER*4,INTENT(IN)  :: ldm, lm
       INTEGER  N
       INTEGER  I, K
       REAL     X, F, DX, H, ALPHA, BETA, GAMMA, ETA
       REAL(8),INTENT(IN)    :: XI(ldm), FI(ldm)
       REAL(8)               :: P2(ldm)
       REAL(8),INTENT(IN)    :: XO(lm)
       REAL(8),INTENT(OUT)   ::FO(lm)
C
C
       N=ldm-1
       Print*,"Entrei SPLINE",ldm, lm,N
       do i=1,ldm
!       Print*,"I,XI,FI",i,XI(i),FI(i)
       enddo
        CALL CUBIC_SPLINE(N, XI, FI, P2)
C
C Find the approximation of the function
C
       do i=1,lm
        X = XO(i)
C
C Find the interval that x resides
        K = 1
        DX = X-XI(1)
        DO WHILE (DX .GE. 0 )
         K = K + 1
         DX = X-XI(K)
        END DO
        if (K.gt.ldm) k=ldm-1
        K = K - 1
C
C Find the value of function f(x)
        DX = XI(K+1) - XI(K)
        ALPHA = P2(K+1)/(6*DX)
        BETA = -P2(K)/(6*DX)
        GAMMA = FI(K+1)/DX - DX*P2(K+1)/6
        ETA = DX*P2(K)/6 - FI(K)/DX
        F = ALPHA*(X-XI(K))*(X-XI(K))*(X-XI(K)) 
     .      +BETA*(X-XI(K+1))*(X-XI(K+1))*(X-XI(K+1)) 
     .      +GAMMA*(X-XI(K))+ETA*(X-XI(K+1))
            FO(i)=F
        WRITE (6, *) X, F
       END DO
      END SUBROUTINE Int_SplineCubica
C
      SUBROUTINE CUBIC_SPLINE (N, XI, FI, P2)
C
C Function to carry out the cubic-spline approximation
C with the second-order derivatives returned.
C
      INTEGER :: I
      INTEGER, INTENT (IN) :: N
      REAL(8), INTENT (IN), DIMENSION (N+1):: XI, FI
      REAL(8), INTENT (OUT), DIMENSION (N+1):: P2
      REAL, DIMENSION (N):: G, H
      REAL, DIMENSION (N-1):: D, B, C
C
C Assign the intervals and function differences
C
      DO I = 1, N
       H(I) = XI(I+1) - XI(I)
       G(I) = FI(I+1) - FI(I)
      END DO
C
C Evaluate the coefficient matrix elements
      DO I = 1, N-1
       D(I) = 2*(H(I+1)+H(I))
       B(I) = 6*(G(I+1)/H(I+1)-G(I)/H(I))
       C(I) = H(I+1)
      END DO
C
C Obtain the second-order derivatives
C
      CALL TRIDIAGONAL_LINEAR_EQ (N-1, D, C, C, B, G)
      P2(1) = 0
      P2(N+1) = 0
      DO I = 2, N 
       P2(I) = G(I-1)
      END DO
      END SUBROUTINE CUBIC_SPLINE
C
      SUBROUTINE TRIDIAGONAL_LINEAR_EQ (L, D, E, C, B, Z)
C
C Functione to solve the tridiagonal linear equation set.
C
      INTEGER, INTENT (IN) :: L
      INTEGER :: I
      REAL, INTENT (IN), DIMENSION (L):: D, E, C, B
      REAL, INTENT (OUT), DIMENSION (L):: Z
      REAL, DIMENSION (L):: Y, W
      REAL, DIMENSION (L-1):: V, T
C
C Evaluate the elements in the LU decomposition
C
      W(1) = D(1)
      V(1)  = C(1)
      T(1)  = E(1)/W(1)
      DO I = 2, L - 1
       W(I) = D(I)-V(I-1)*T(I-1)
       V(I) = C(I)
       T(I) = E(I)/W(I)
      END DO
      W(L) = D(L)-V(L-1)*T(L-1)
C
C Forward substitution to obtain y
C
      Y(1) = B(1)/W(1)
      DO I = 2, L
       Y(I) = (B(I)-V(I-1)*Y(I-1))/W(I)
      END DO
C
C Backward substitution to obtain z
      Z(L) = Y(L)
      DO I = L-1, 1, -1
       Z(I) = Y(I) - T(I)*Z(I+1)
      END DO
      END SUBROUTINE TRIDIAGONAL_LINEAR_EQ
